#!/usr/bin/env python

# This script calculates statistics of the percentage of translation per
# language for all languages in Ubuntu by calling all involved functions in the
# right order.
#
# Author: David Planella <david.planella@ubuntu.com>
#
# Copyright (c) 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import urllib
import os
import subprocess
from optparse import OptionParser
from common import utils
from ul10n_stats import ul10n_iso_pots_get
from ul10n_stats import ul10n_stats_calc
from ul10n_stats import ul10n_custom

DATA_DIR = 'data'
CONFIG_DIR = 'config'
STATS_URL = 'http://people.canonical.com/~danilo/ubuntu'

def options_parse():
    """Parses the command line and config file options"""

    settings = utils.get_settings()

    USAGE = "%prog [OPTIONS]"

    DESCRIPTION = """Lists all translation templates and source packages in the LiveCD for the
    given distro version."""

    parser = OptionParser(usage = USAGE, description = DESCRIPTION)
    parser.add_option("-p",
                    "--no-priorities",
                    dest="no_priorities",
                    action="store_true",
                    default=False,
                    help="Disables using priorities to generate blacklist.")

    parser.add_option("-s",
                    "--source-package-stats",
                    dest="pkg_stats",
                    default='{0}/{1}-package-stats.log.gz'.format(DATA_DIR, settings['distro_codename']),
                    help="Specify the file containing the source package statistics")

    parser.add_option("-b",
                    "--template-blacklist",
                    dest="blacklist",
                    default='config/blacklist.conf',
                    help="Specify the file containing the templates to blacklist during the calculation of statistics")
# from here down are args for custom options. they all required -c.
    parser.add_option("-c",
                    "--custom",
                    dest="custom",
                    action="store_true",
                    default=False,
                    help="Turn on custom (non-Ubuntu) mode")

    parser.add_option("-l",
                    "--languages-file",
                    dest="languages_file",
                    default=None,
                    help="File with languages for report.")

    parser.add_option("-n",
                    "--custom-name",
                    dest="custom_name",
                    default=None,
                    help="Name of custom project. Requires -c argument.")

    parser.add_option("-m",
                    "--manifest-file",
                    dest="manifest_file",
                    default=None,
                    help="Manifest file to read bin pkgs from and on which stats are based. Requires -c argument.")

    parser.add_option("-d",
                    "--domains-file",
                    dest="domains_file",
                    default=None,
                    help="File to read domains from and on which stats are based. Requires -c argument.")

    parser.add_option("-r",
                    "--root-dir",
                    dest="root_dir",
                    default=None,
                    help="Path to root dir of custom system from which to get domains. Requires -c argument.")
    parser.add_option("-f",
                    "--no-file",
                    dest="no_file",
                    action="store_true",
                    default=False,
                    help="Disables using conf file blacklist.")

    (options, args) = parser.parse_args()

    return options, settings


def get_raw_data(distro_codename):

    print >>sys.stderr, "Fetching stats data..."
    for stats_file in ('potemplate', 'package'):
        url = '{0}/{1}-{2}-stats.log.gz'.format(STATS_URL, distro_codename, stats_file)
        path = os.path.join(DATA_DIR, os.path.basename(url))
        urllib.urlretrieve(url, path)


def get_working_templates(options, settings):

    distro_codename = settings['distro_codename']
    distro_id = settings['distro_id']
    TEMPLATES_FILE = '{0}/{1}-potemplate-stats.log.gz'.format(DATA_DIR, distro_codename)

    print >>sys.stderr, "Fetching working templates set..."

    src_pkgs_not_found = list() 
    
    # if normal ubuntu mode, get src pkg list from seeds, else from custom
    if not options.custom:
        distro_src_pkgs = ul10n_iso_pots_get.get_src_pkgs_from_seeds(distro_id,
                                                                    distro_codename)
        working_pots = ul10n_iso_pots_get.apply_valid_pots_filter(TEMPLATES_FILE, distro_src_pkgs)

    else:
        domain_src, domain_linenum, domain_template, src_domains, lines = ul10n_iso_pots_get.get_dictionaries(TEMPLATES_FILE)
        if options.manifest_file:
            binary_pkgs = get_bin_pkgs_from_manifest(options.manifest_file)
            src_pkgs, not_found = get_src_pkgs_from_binary_pkgs(binary_pkgs)
            domains = list()
            for src_pkg in src_pkgs:
                if src_pkg in src_domains.keys():
                    for domain in src_domains[src_pkg]:
                        domains.append(domain)
                else:
                    src_pkgs_not_found.append(src_pkg)
                #TODO save src pkgs not found. this shows what the trans stats DON'T include
        elif options.domains_file:
            if not os.path.exists(options.domains_file):
                print >> sys.stderr, "Error: domains_file not found. Stopping."
                sys.exit(1)
            f = open(options.domains_file, 'r')
            domains = f.read().split('\n')
            f.close()
        else:
            if options.root_dir:
                r_dir = options.root_dir
            else:
                r_dir = '/'
            domains = ul10n_custom.get_domains(r_dir)
        #for domain in domains: print >> sys.stderr, domain
        working_pots = list()
        invalid = list()
        for domain in domains:
            if domain in domain_src.keys():
                line = lines[domain_linenum[domain]].rstrip().lstrip()
                working_pots.append(line)
        working_pots.sort()

    return working_pots, src_pkgs_not_found

def get_src_pkgs_from_binary_pkgs(binaries):
    not_found = list()
    src_pkgs = list()
    for tup in binaries:
        cmd = ['apt-cache', 'showsrc', tup[0]]
        try:
            res = subprocess.Popen(cmd, cwd="./", stdout=subprocess.PIPE ).communicate()
        except:
            print >>sys.stderr, "Error: subprocess exception. stopping."
            sys.exit(1)
            continue
        src_pkg = res[0][res[0].find(" ") + 1:res[0].find("\n")]
        if len(src_pkg) == 0:
            not_found.append(tup[0])
            continue
        if src_pkg not in src_pkgs:
            src_pkgs.append(src_pkg)
        src_pkgs.sort()

        #print >>sys.stderr, "src pkg: %(res)s" % {'res':src_pkg}
    return src_pkgs, not_found

def get_bin_pkgs_from_manifest(path):
    if not os.path.exists(path):
        print >> sys.stderr, "Error: " + path + "does not exist. Stoppping."
        sys.exit(1)
    f = open(path, 'r')
    lines = f.readlines()
    f.close()

    binaries = list(tuple())
    for line in lines:
        items = line.split()
        binaries.append((items[0],items[1]))
    return binaries


def get_not_blacklisted(options, settings, working):

    working, blacklisted, blacklisted_priority, blacklisted_file = ul10n_iso_pots_get.apply_blacklist_filter(options, settings, working)

    return working, blacklisted

def log_template(the_set, name, options, settings):

    if options.custom:
        f_name = "custom-" + name
    else:
        f_name = "ubuntu-" + name
    f = open(os.path.join(os.getcwd(), "reports", f_name), 'w')
    the_list_ = list(the_set)
    the_list_.sort()
    for line  in the_list_:
        f.write(line + '\n')
    f.flush()
    f.close()

    return

def log_blacklist(the_list, name, options, settings):

    if options.custom:
        f_name = "custom-blacklist-" + name
    else:
        f_name = "ubuntu-blacklist-" + name

    # write used blacklist to file in data dir
    f = open(os.path.join("reports", f_name), 'w')
    l = list(the_list)
    l.sort()
    for item in l:
        f.write(item[0] + ' | ' + item[1] + '\n')
    f.flush()
    f.close()

    return

def log(blacklisted, blacklisted_priority, blacklisted_file, superset, working, 
        src_pkgs_not_found, options, settings):
    # write blaclisted files
    log_blacklist(blacklisted_file, "file", options, settings)
    log_blacklist(blacklisted_priority, "priority", options, settings)
    log_blacklist(blacklisted, "all", options, settings)

    log_template(superset, "superset", options, settings)
    log_template(working, "working", options, settings)
    if options.manifest_file:
        log_template(src_pkgs_not_found, "src_pgks_not_found", options, settings)

    return


def main():

    # Get the command line options and settings from the config file
    options, settings = options_parse()

    # Fetch the raw data export files to be used as the source for the
    # calculation
    get_raw_data(settings['distro_codename'])

    # Compile a list of working templates to be considered for the calculation.
    superset, src_pkgs_not_found = get_working_templates(options, settings)

    # Apply blacklists 
    working, blacklisted, blacklisted_priority, blacklisted_file = ul10n_iso_pots_get.apply_blacklist_filter(options, settings, superset)
    #working, blacklisted, blacklisted_proirity, blacklisted_file = get_not_blacklisted(options, settings, working)

    # write log files
    log(blacklisted, blacklisted_priority, blacklisted_file, superset, working, src_pkgs_not_found, options, settings)

    # Calculate the unified, per language translation statistics from the
    # working templates set and output them in a report
    ul10n_stats_calc.calculate_stats(options, settings, working)

if __name__ == "__main__":
    main()
    sys.exit(0)
